﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace AppointmentCommon
{
    public static class AppointmentDatabase
    {
        public static List<AppointmentModel> AppointmentTable { get; private set; }

        private static int _sessionLengths;
        private static TimeSpan _dayStart;
        private static TimeSpan _dayEnd;

        public static void Initialise(int sessionLengths, TimeSpan dayStart, TimeSpan dayEnd)
        {
            _sessionLengths = sessionLengths;
            _dayStart = dayStart;
            _dayEnd = dayEnd;
            AppointmentTable = new List<AppointmentModel>(AppointmentGeneration.GenerateRandomAppointments(sessionLengths, DateTime.Today, dayStart, dayEnd));
        }

        public static void Reset()
        {
            Initialise(_sessionLengths, _dayStart, _dayEnd);
        }

        public static IEnumerable<AppointmentModel> GetAllAppointments()
        {
            return AppointmentTable.ToList();
        }

        public static AppointmentModel GetAppointmentByID(int id)
        {
            return AppointmentTable.FirstOrDefault(x => x.ID == id);
        }

        public static IEnumerable<AppointmentModel> GetAppointmentsBetween(DateTime t1, DateTime t2)
        {
            return AppointmentTable.Where(x => x.Due >= t1 && x.Due.Add(x.Duration) <= t2);
        }

        public static void InsertAppointment(AppointmentModel appt)
        {
            AppointmentTable.Add(appt);
        }

        public static void UpdateAppointment(AppointmentModel updatedAppointment)
        {
            var appt = AppointmentTable.Single(x => x.ID == updatedAppointment.ID);
            appt.Due = updatedAppointment.Due;
            appt.Duration = updatedAppointment.Duration;
            appt.ApptName = updatedAppointment.ApptName;
            appt.Notes = updatedAppointment.Notes;
        }

        public static void DeleteAppointment(int deleteApptID)
        {
            AppointmentTable.Remove(AppointmentTable.Single(x => x.ID == deleteApptID));
        }
    }
}
