﻿using System;
using System.ComponentModel;
using AppointmentCommon.Annotations;

namespace AppointmentCommon
{
    public class AppointmentViewModel : INotifyPropertyChanged
    {
        private int _id;
        public int ID
        {
            get { return _id; }
            set
            {
                _id = value;
                IsDirty = true;
                OnPropertyChanged("ID");
            }
        }

        private string _apptName;
        public string ApptName
        {
            get { return _apptName; }
            set
            {
                _apptName = value;
                IsDirty = true;
                OnPropertyChanged("ApptName");
            }
        }

        private DateTime _due;
        public DateTime Due
        {
            get { return _due; }
            set
            {
                _due = value;
                IsDirty = true;
                OnPropertyChanged("Due");
            }
        }

        private TimeSpan _duration;
        public TimeSpan Duration
        {
            get { return _duration; }
            set
            {
                _duration = value;
                IsDirty = true;
                OnPropertyChanged("Duration");
            }
        }

        private string _notes;
        public string Notes
        {
            get { return _notes; }
            set
            {
                _notes = value;
                IsDirty = true;
                OnPropertyChanged("Notes");
            }
        }

        private bool _isDirty;
        public bool IsDirty
        {
            get { return _isDirty; }
            private set
            {
                if (value == _isDirty) return;

                _isDirty = value;
                OnPropertyChanged("IsDirty");
            }
        }

        public AppointmentModel OriginalAppointment { get; private set; }

        public AppointmentViewModel(AppointmentModel appt)
        {
            OriginalAppointment = appt;
            IsDirty = false;
            ID = appt.ID;
            ApptName = appt.ApptName;
            Due = appt.Due;
            Duration = appt.Duration;
            Notes = appt.Notes;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
