﻿using System;
using System.ComponentModel;
using AppointmentCommon.Annotations;

namespace AppointmentCommon
{
    public class TimeSlotViewModel : INotifyPropertyChanged
    {
        private DateTime _time;
        public DateTime Time
        {
            get { return _time; }
            set
            {
                _time = value;
                OnPropertyChanged("Time");
                OnPropertyChanged("TimeString");
            }
        }

        public string TimeString
        {
            get { return _time.ToString("h:mm tt"); }
        }

        public string NameString
        {
            get
            {
                return Appointment == null ? null : Appointment.ApptName;
            }
        }

        public string IDString
        {
            get
            {
                return Appointment == null ? null : Appointment.ID.ToString();
            }
        }

        public string NotesString
        {
            get
            {
                return Appointment == null ? null : Appointment.Notes;
            }
        }

        private AppointmentViewModel _appointment;
        public AppointmentViewModel Appointment
        {
            get { return _appointment; }
            set
            {
                _appointment = value;
                OnPropertyChanged("Appointment");
                OnPropertyChanged("NameString");
                OnPropertyChanged("NotesString");
                OnPropertyChanged("IDString");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
