﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using System.Timers;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;
using AppointmentCommon;
using C1.WPF.FlexGrid;

namespace TestAppointmentsComponentOne
{
    public partial class MainWindow
    {

        private static IEnumerable<TimeSlotViewModel> CreateTimeSlots(int sessionLength, TimeSpan dayStart, TimeSpan dayEnd, IEnumerable<AppointmentModel> appointments)
        {
            var appointmentModels = appointments as AppointmentModel[] ?? appointments.ToArray();
            var currentTime = dayStart;
            var timeslotViewModels = new List<TimeSlotViewModel>();
            while (currentTime < dayEnd)
            {
                var appointmentsCurrent = appointmentModels.Where(x => x.Due.TimeOfDay >= currentTime && x.Due.TimeOfDay < currentTime.Add(new TimeSpan(0, 15, 0)) ||
                                                                       x.Due.TimeOfDay < currentTime && x.Due.TimeOfDay.Add(x.Duration) > currentTime && x.Due.TimeOfDay.Add(x.Duration) <= currentTime.Add(new TimeSpan(0, 15, 0))).ToList();
                if (!appointmentsCurrent.Any())
                {
                    timeslotViewModels.Add(new TimeSlotViewModel(){ Appointment = null, Time = DateTime.Today.Add(currentTime) });
                }
                else
                {
                    timeslotViewModels.AddRange(appointmentsCurrent.Select(appt => new TimeSlotViewModel{ Appointment = new AppointmentViewModel(appt), Time = DateTime.Today.Add(currentTime) }));
                }

                currentTime = currentTime.Add(new TimeSpan(0,sessionLength,0));
            }
            return timeslotViewModels;
        }

        private AppointmentBookViewModel _vm;

        public MainWindow()
        {

            InitializeComponent();
            var hourEnd = 16 + +Helpers.RandomInt(3);
            AppointmentDatabase.Initialise(15, new TimeSpan(8, 0, 0), new TimeSpan(hourEnd, 0, 0));
            CreateAndSet(hourEnd);

            var t = new Timer{
                                 AutoReset = true,
                                 Interval = 1000 * 5
                             };
            t.Elapsed += (o, args) =>
                {
                    Refresh();
                };
            t.Start();
        }

        private void CreateAndSet(int hourEnd)
        {
            _vm = new AppointmentBookViewModel();
            var timeSlots = CreateTimeSlots(15, new TimeSpan(8, 0, 0), new TimeSpan(hourEnd, 0, 0), AppointmentDatabase.GetAllAppointments());
            foreach (var timeSlot in timeSlots)
            {
                _vm.DaySessionDataList.Add(timeSlot);
            }
            DataContext = _vm;
        }



        private void FlexGrid_PreviewMouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            var ht = FlexGridDaySession.HitTest(e.GetPosition(FlexGridDaySession));
            if (ht.CellType == CellType.Cell)
            {
                FlexGridDaySession.Select(ht.Row, ht.Column);
            }
        }

        private void FlexGridDaySession_OnItemsSourceChanged(object sender, EventArgs e)
        {
            Refresh();
        }

        private void Refresh()
        {
            var timer = new Timer(50) { AutoReset = false };
            timer.Elapsed += (o, args) =>
                {
                    Dispatcher.BeginInvoke(DispatcherPriority.ApplicationIdle, (Action)delegate
                        {
                            //FlexGridDaySession.AutoSizeRows(0, FlexGridDaySession.Rows.Count - 1, 25, true);
                            FlexGridDaySession.AutoSizeColumns(0, FlexGridDaySession.Columns.Count - 1, 6, true, false);
                            FlexGridDaySession.AutoSizeRows(0, FlexGridDaySession.Rows.Count - 1, 0, true);
                        });
                };
            timer.Start();
        }

        private void FlexGridDaySession_OnLoaded(object sender, RoutedEventArgs e)
        {
            Dispatcher.BeginInvoke((Action)delegate
                {
                    FlexGridDaySession.Invalidate();
                    foreach (var col in FlexGridDaySession.Columns.Where(col => col.ColumnName != ""))
                    {
                        col.AllowMerging = true;
                    }
                });
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            for (var i = 0; i < 1; i++)
            {
                AppointmentDatabase.Reset();
                var timeSlots = CreateTimeSlots(15, new TimeSpan(8, 0, 0), new TimeSpan(16 + Helpers.RandomInt(3), 0, 0), AppointmentDatabase.GetAllAppointments());
                _vm.DaySessionDataList.Clear();
                foreach (var timeSlot in timeSlots)
                {
                    _vm.DaySessionDataList.Add(timeSlot);
                }
            }
            FlexGridDaySession.ItemsSource = _vm.DaySessionDataList;
            Refresh();
        }
    }
}
